/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.server;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Level;

import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementException;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.ListInstance.MInstance;
import org.ow2.orchestra.console.GWTConsole.client.ListInstanceActivity.MInstanceActivity;
import org.ow2.orchestra.console.GWTConsole.client.ListProcess.MProcess;
import org.ow2.orchestra.console.GWTConsole.client.ListProcessActivity.MProcessActivity;
import org.ow2.orchestra.facade.AccessorUtil;
import org.ow2.orchestra.facade.InstanceManagementAPI;
import org.ow2.orchestra.facade.ManagementAPI;
import org.ow2.orchestra.facade.QuerierDefinitionAPI;
import org.ow2.orchestra.facade.QuerierRuntimeAPI;
import org.ow2.orchestra.facade.def.ActivityDefinition;
import org.ow2.orchestra.facade.def.ProcessDefinition;
import org.ow2.orchestra.facade.exception.ActivityInstanceNotFoundException;
import org.ow2.orchestra.facade.exception.ActivityNotFoundException;
import org.ow2.orchestra.facade.exception.InstanceNotFoundException;
import org.ow2.orchestra.facade.exception.ProcessNotFoundException;
import org.ow2.orchestra.facade.runtime.ActivityInstance;
import org.ow2.orchestra.facade.runtime.ProcessInstance;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.util.Misc;


/**
 * Server that manage the relation between the client and the orchestra server.
 *
 * @author Mathieu Crassin
 */
@SuppressWarnings("serial")
public class OrchestraManagementServlet extends RemoteServiceServlet
                                        implements OrchestraManagementServices {

  /**
   * Service url and Object Name to connect to Orchestra
   */
  protected String serviceUrl = null;
  protected String objectName = null;

  /**
   * APIs used to get information on orchestra
   */
  protected InstanceManagementAPI instanceManagementAPI = null;
  protected QuerierDefinitionAPI  querierDefinitionAPI = null;
  protected QuerierRuntimeAPI     querierRuntimeAPI = null;
  protected ManagementAPI         managementAPI = null;

  /**
   * Constructor of the class. Used to fill the service url and object name fields when
   * connection to orchestra is initialized
   *
   * @author Jean-Yves Goubet
   */
  public OrchestraManagementServlet() throws OrchestraManagementException {
    this.getOrchestraProperties();
  }

  /**
   * Get the file named filename on the disk and deploy it to orchestra.
   *
   * @author mathieu
   * @param filename name of the file to deploy
   * @return int return whether or not the function fail
   *          -1 <-> Error
   *          0  <-> Success
   * @throws MyOrchestraException : to get some orchestra exception and transmit
   *                                them to the client side
   */
  public final void deployToOrchestra(final String filename) throws OrchestraManagementException {
    Misc.log(Level.INFO,"deployToOrchestra: RPC call start");

    // we get the data of the file
    byte []buffer = null;
    buffer = this.getContentFile(filename);

    // check if the API fields are null or not
    if (this.managementAPI == null) {
      this.getAPIs();
    }

    // then we deploy the data
    ProcessDefinition process = null;
    try {
      // deploy these data to orchestra
      process = this.managementAPI.deployBar(buffer);

    } catch (final Exception ex) {
      Misc.log(Level.SEVERE,"OrchestraManagement - deployToOchestra :"
          + "Unable to deploy file on orchestra server !\n" + Misc.getStackTraceFrom(ex));
      throw new OrchestraManagementException("Unable to deploy file on orchestra server !",
                                             "OrchestraManagement - deployToOchestra : \n" + Misc.getStackTraceFrom(ex));
    }

    // check the result
    if (process != null) {
      Misc.log(Level.INFO,"deployToOrchestra: File " + filename + " deployed");
      Misc.log(Level.INFO,"name: " + process.getName());
      Misc.log(Level.INFO,"deployed date: " + process.getDeployedDate());
    } else {
      Misc.log(Level.SEVERE,"OrchestraManagement - deployToOchestra : Fail to deploy " + filename);
      throw new OrchestraManagementException("Error while deploy to orchestra server !",
                                             "OrchestraManagement - deployToOrchestra : Fail to deploy " + filename);
    }
  }


  /**
   * Undeploy a process from the orchestra server thanks to its UUID.
   *
   * @author Mathieu Crassin
   * @param processUUID : Information about the process to undeploy.
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                them to the client side
   */
  public final void undeployFromOrchestra(final String processUUID) throws OrchestraManagementException {
    // check if the API fields are null or not
    if (this.managementAPI == null) {
      this.getAPIs();
    }

    // then modify the input parameter.
    final ProcessDefinitionUUID uuid = new ProcessDefinitionUUID(processUUID);

    // then, call the function to undeploy the process
    boolean res = false;
    try {
      res = this.managementAPI.undeploy(uuid);

    } catch (final Exception ex) {
      Misc.log(Level.SEVERE,"OrchestraManagement - undeployFromOchestra :"
          + "Fail to undeploy process with uuid = " + processUUID + "\n" + Misc.getStackTraceFrom(ex));
      throw new OrchestraManagementException("Fail to undeploy process with uuid = " + processUUID,
          "OrchestraManagement - undeployFromOrchestra : \n" + Misc.getStackTraceFrom(ex));
    }
    // check return result
    if (res) {
      Misc.log(Level.INFO,"OrchestraManagement - undeployFromOchestra : File with uuid = " + processUUID + " undeployed");
    } else {
      Misc.log(Level.SEVERE,"OrchestraManagement - undeployFromOchestra :"
          + "Fail to undeploy process with uuid = " + processUUID);
      throw new OrchestraManagementException("Fail to undeploy process with uuid = " + processUUID,
          "OrchestraManagement - undeployFromOrchestra : Fail to undeploy process with uuid = " + processUUID);
    }
  }

  /**
   * Retires a process from the orchestra server thanks to its UUID.
   *
   * @author Guillaume Porcher
   * @param processUUID : Information about the process to retire.
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                them to the client side
   */
  public final void retireProcess(final String processUUID) throws OrchestraManagementException {
    // check if the API fields are null or not
    if (this.managementAPI == null) {
      this.getAPIs();
    }

    // then modify the input parameter.
    final ProcessDefinitionUUID uuid = new ProcessDefinitionUUID(processUUID);

    // then, call the function to undeploy the process
    boolean res = false;
    try {
      res = this.managementAPI.retire(uuid);

    } catch (final Exception ex) {
      Misc.log(Level.SEVERE,"OrchestraManagement - retireProcess :"
          + "Fail to retire process with uuid = " + processUUID + "\n" + Misc.getStackTraceFrom(ex));
      throw new OrchestraManagementException("Fail to retire process with uuid = " + processUUID,
          "OrchestraManagement - retireProcess : \n" + Misc.getStackTraceFrom(ex));
    }
    // check return result
    if (res) {
      Misc.log(Level.INFO,"OrchestraManagement - retireProcess : Process with uuid = " + processUUID + " retired");
    } else {
      Misc.log(Level.SEVERE,"OrchestraManagement - retireProcess :"
          + "Fail to retire process with uuid = " + processUUID);
      throw new OrchestraManagementException("Fail to retire process with uuid = " + processUUID,
          "OrchestraManagement - retireProcess : Fail to retire process with uuid = " + processUUID);
    }
  }

  /**
   * Activates a process from the orchestra server thanks to its UUID.
   *
   * @author Guillaume Porcher
   * @param processUUID : Information about the process to activate.
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                them to the client side
   */
  public final void activateProcess(final String processUUID) throws OrchestraManagementException {
    // check if the API fields are null or not
    if (this.managementAPI == null) {
      this.getAPIs();
    }

    // then modify the input parameter.
    final ProcessDefinitionUUID uuid = new ProcessDefinitionUUID(processUUID);

    // then, call the function to undeploy the process
    boolean res = false;
    try {
      res = this.managementAPI.activate(uuid);

    } catch (final Exception ex) {
      Misc.log(Level.SEVERE,"OrchestraManagement - activateProcess :"
          + "Fail to activate process with uuid = " + processUUID + "\n" + Misc.getStackTraceFrom(ex));
      throw new OrchestraManagementException("Fail to activate process with uuid = " + processUUID,
          "OrchestraManagement - activateProcess : \n" + Misc.getStackTraceFrom(ex));
    }
    // check return result
    if (res) {
      Misc.log(Level.INFO,"OrchestraManagement - activateProcess : Process with uuid = " + processUUID + " activated");
    } else {
      Misc.log(Level.SEVERE,"OrchestraManagement - activateProcess :"
          + "Fail to activate process with uuid = " + processUUID);
      throw new OrchestraManagementException("Fail to activate process with uuid = " + processUUID,
          "OrchestraManagement - activateProcess : Fail to activate process with uuid = " + processUUID);
    }
  }

  /**
   * This method gets the processes on orchestra.
   *
   * @author Jean-Yves Goubet
   * @return mProcessList : the list of processes
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   *
   */
  public final List<MProcess> getProcesses() throws OrchestraManagementException {
    final List<MProcess> mProcessList = new ArrayList<MProcess>();
    List<ProcessDefinition> processDefinitionList = null;

    // check if the API fields are null or not
    if (this.querierDefinitionAPI == null) {
      this.getAPIs();
    }

    // get the process list
    try {
      processDefinitionList = this.querierDefinitionAPI.findProcessDefinitions();
    } catch (final Exception ex) {
      Misc.log(Level.SEVERE, "Error while sending request to orchestra server !");
      throw new OrchestraManagementException("Error while sending request to orchestra server !",
          "OrchestraManagement - getProcesses : \n" + Misc.getStackTraceFrom(ex));
    }
    if (processDefinitionList != null) {
      for (final ProcessDefinition p : processDefinitionList) {
        mProcessList.add(new MProcess(p.getUUID().toString(), p.getName(), p.getNameSpace(), p.getVersion(), p.getDeployedDate(),
            p.getUndeployedDate(), p.getState().toString().toLowerCase()));
      }
    } else {
      Misc.log(Level.SEVERE,"OrchestraManagement - getProcesses :"
          + "Unable to get process list on orchestra server !\nThe return result of orchestra is null.");
      throw new OrchestraManagementException("Unable to get process list on orchestra server !",
                                             "OrchestraManagement - getProcesses : The return result of orchestra is null.");
    }
    return mProcessList;
  }

  /**
   * This method gets the processes on orchestra.
   *
   * @author Jean-Yves Goubet
   * @return mProcessList : the list of processes
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   *
   */
  public final MProcess getProcess(final String processUUID) throws OrchestraManagementException {
    MProcess p = null;
    ProcessDefinition processDef = null;

    // check if the API fields are null or not
    if (this.querierDefinitionAPI == null) {
      this.getAPIs();
    }

    // get the process list
    try {
      processDef = this.querierDefinitionAPI.getProcessDefinition(new ProcessDefinitionUUID(processUUID));
    } catch (final Exception ex) {
      Misc.log(Level.SEVERE, "Error while sending request to orchestra server !");
      throw new OrchestraManagementException("Error while sending request to orchestra server !",
          "OrchestraManagement - getProcesses : \n" + Misc.getStackTraceFrom(ex));
    }

    if (processDef != null) {
      p = new MProcess(
          processDef.getUUID().toString(),
          processDef.getName(),
          processDef.getNameSpace(),
          processDef.getVersion(),
          processDef.getDeployedDate(),
          processDef.getUndeployedDate(),
          processDef.getState().toString().toLowerCase());
    } else {
      Misc.log(Level.SEVERE,"OrchestraManagement - getProcess :"
          + "Unable to get process information on orchestra server !\nThe return result of orchestra is null.");
      throw new OrchestraManagementException("Unable to get process information on orchestra server !",
                                             "OrchestraManagement - getProcess : The return result of orchestra is null.");
    }

    return p;
  }

  /**
   * This method gets the instances of process on orchestra.
   *
   * @author Jean-Yves Goubet
   * @param processId : the id of the process when specified
   * @return mInstanceList : the list of instances
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   *
   */
  public final List<MInstance> getInstances(final String processId) throws OrchestraManagementException {
    final List<MInstance> mInstanceList = new ArrayList<MInstance>();
    Set<ProcessInstance> processInstanceSet = null;

    // check if the API fields are null or not
    if (this.querierRuntimeAPI == null) {
      this.getAPIs();
    }

    // we ask for all instances
    if (processId == null) {
      //case when processId not specified
      try {
        processInstanceSet = this.querierRuntimeAPI.findProcessInstances();
      } catch (final Exception ex) {
        Misc.log(Level.SEVERE, "Error while accessing instances list !\n");
        throw new OrchestraManagementException("Error while accessing instances list !",
            "OrchestraManagement - getInstances : \n" + Misc.getStackTraceFrom(ex));
      }
    }
    // we ask for only the instances of the specified process
    else {
      try {
        processInstanceSet = this.querierRuntimeAPI.findProcessInstances(new ProcessDefinitionUUID(processId));
      } catch (final ProcessNotFoundException e) {
        Misc.log(Level.SEVERE, "Error while accessing instances list of process " + processId +
            " !\nThe process was not found on orchestra");
        throw new OrchestraManagementException("Error while accessing instances list of process " + processId + " !",
            "OrchestraManagement - getInstances : The process was not found on orchestra\n" + Misc.getStackTraceFrom(e));
      } catch (final Exception e) {
        Misc.log(Level.SEVERE, "Error while accessing instances list of process " + processId + " !\n");
        throw new OrchestraManagementException("Error while accessing instances list of process " + processId + " !",
            "OrchestraManagement - getInstances :\n" + Misc.getStackTraceFrom(e));
      }
    }

    // check the result
    if (processInstanceSet != null) {
      // the correspondence list that permit to avoid asking the same things many times to orchestra
      final List<ProcessUuidNName> correspondenceList = new ArrayList<ProcessUuidNName>();

      // create the mInstanceList object from ProcessInstance set
      for (final ProcessInstance p : processInstanceSet) {

        // for each instance, we get the process name (thanks to the process ID)
        final String processName = this.searchProcessName(correspondenceList, p.getProcessDefinitionUUID());

        // and we create the MInstance object corresponding to the ProcessInstance object
        mInstanceList.add(new MInstance(p.getProcessInstanceUUID().toString(),
            p.getProcessDefinitionUUID().toString(),
            processName,
            p.getState().toString().toLowerCase(),
            p.getStartedDate(),
            p.getProcessInstanceStateUpdate().get(p.getProcessInstanceStateUpdate().size() - 1).getDate()));
      }
    } else {
      Misc.log(Level.SEVERE,"OrchestraManagement - getInstances :"
          + "Unable to get instances list on orchestra server !\nThe return result of orchestra is null.");
      throw new OrchestraManagementException("Unable to get instances list on orchestra server !",
          "OrchestraManagement - getInstances : The return result of orchestra is null.");
    }

    return mInstanceList;
  }


  /**
   * This method gets the info on a unique instance
   *
   * @author Jean-Yves Goubet
   * @param instanceId : the id of the instance
   */
  public final MInstance getInstance(final String instanceId) throws OrchestraManagementException {
    MInstance i = null;
    ProcessInstance processInstance = null;

    if (instanceId == null || instanceId.equals("")) {
      return null;
    }

    // check if the API fields are null or not
    if (this.querierRuntimeAPI == null) {
      this.getAPIs();
    }

    //
    try {
      processInstance = this.querierRuntimeAPI.getProcessInstance(new ProcessInstanceUUID(instanceId));
    } catch (final InstanceNotFoundException e) {
      Misc.log(Level.SEVERE, "Error while accessing instance information ( id = " + instanceId
          + " ) !\nThe instance was not found on orchestra");
      throw new OrchestraManagementException("Error while accessing instance information ( id = " + instanceId + " ) !",
          "OrchestraManagement - getInstance : The instance was not found on orchestra\n" + Misc.getStackTraceFrom(e));
    } catch (final Exception e) {
      Misc.log(Level.SEVERE, "Error while accessing instance information ( id = " + instanceId + " ) !");
      throw new OrchestraManagementException("Error while accessing instance information ( id = " + instanceId + " ) !",
          "OrchestraManagement - getInstance :\n" + Misc.getStackTraceFrom(e));
    }

    //
    if (processInstance != null) {
      final String processName = this.searchProcessName(null, processInstance.getProcessDefinitionUUID());
      i = new MInstance(processInstance.getUUID().toString(),
          processInstance.getProcessDefinitionUUID().toString(), processName,
          processInstance.getState().toString().toLowerCase(), processInstance.getStartedDate(),
          processInstance.getProcessInstanceStateUpdate().get(processInstance.
              getProcessInstanceStateUpdate().size() - 1).getDate());
    } else {
      Misc.log(Level.SEVERE,"OrchestraManagement - getInstance :"
          + "Unable to get instance information on orchestra server !\nThe return result of orchestra is null.");
      throw new OrchestraManagementException("Unable to get instance information on orchestra server !",
          "OrchestraManagement - getInstance : The return result of orchestra is null.");
    }

    return i;
  }


  /**
   * This method gets the info of an activity
   *
   * @author Jean-Yves Goubet
   * @param activityId : the id of the activity
   * @return MInstanceActivity : the instance activity info
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  public final MInstanceActivity getActivity(final String activityId) throws OrchestraManagementException {
    MInstanceActivity instanceActivity = null;
    ActivityInstance activityInstance = null;
    String activityName = null;

    // check if the API fields are null or not
    if (this.querierRuntimeAPI == null) {
      this.getAPIs();
    }

    //
    try {
      activityInstance = this.querierRuntimeAPI.getActivityInstance(new ActivityInstanceUUID(activityId));
      activityName = this.querierDefinitionAPI.getActivityDefinition(activityInstance.getDefinitionUUID()).getName();
    } catch (final ActivityNotFoundException e) {
      Misc.log(Level.SEVERE,"OrchestraManagement - getActivity :"
          + "Unable to get activity information on orchestra server !\n");
      throw new OrchestraManagementException("Unable to get activity information on orchestra server !",
                                             "OrchestraManagement - getActivity : Activity not found !");
    } catch (final ActivityInstanceNotFoundException e) {
      Misc.log(Level.SEVERE,"OrchestraManagement - getActivity :"
          + "Unable to get activity information on orchestra server !\nInstance activity not found !");
      throw new OrchestraManagementException("Unable to get activity information on orchestra server !",
                                             "OrchestraManagement - getActivity : Instance activity not found !");
    } catch (final Exception e) {
      Misc.log(Level.SEVERE,"OrchestraManagement - getActivity :"
          + "Unable to get activity information on orchestra server !\nUnknown error !");
      throw new OrchestraManagementException("Unable to get activity information on orchestra server !",
                                             "OrchestraManagement - getActivity : Unknown error !");
    }

    //
    if (activityName != null) {
      instanceActivity = new MInstanceActivity(activityInstance.getUUID().toString(), activityName, activityInstance.getType().toString(),
          activityInstance.getState().toString(), activityInstance.getStartedDate(), activityInstance.getEndedDate());
    } else {
      Misc.log(Level.SEVERE,"OrchestraManagement - getActivity :"
          + "Unable to get activity information on orchestra server !\nThe return result of orchestra is null.");
      throw new OrchestraManagementException("Unable to get activity information on orchestra server !",
                                             "OrchestraManagement - getActivity : The return result of orchestra is null.");
    }
    return instanceActivity;
  }


  /**
   * Suspend the instance that have the specified Id on the orchestra server.
   *
   * @author Mathieu Crassin
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  public final void suspendInstance(final String instanceId) throws OrchestraManagementException {
    // check if the API fields are null or not
    if (this.instanceManagementAPI == null) {
      this.getAPIs();
    }

    try {
      final ProcessInstanceUUID inst = new ProcessInstanceUUID(instanceId);
      this.instanceManagementAPI.suspend(inst);
    } catch (final InstanceNotFoundException e) {
      Misc.log(Level.SEVERE,"OrchestraManagement - suspendInstance :"
          + "Error, instance " + instanceId + " not found !\n" + Misc.getStackTraceFrom(e));
      throw new OrchestraManagementException("Error while suspending the instance with uuid " + instanceId + " !",
          "OrchestraManagement - suspendInstance : Error, instance not found !\n" + Misc.getStackTraceFrom(e));
    } catch (final Exception ex) {
      Misc.log(Level.SEVERE,"OrchestraManagement - suspendInstance :"
          + "Error while suspending instance : " + instanceId + " !\n" + Misc.getStackTraceFrom(ex));
      throw new OrchestraManagementException("Error while suspending the instance with uuid " + instanceId + " !",
          "OrchestraManagement - suspendInstance : \n" + Misc.getStackTraceFrom(ex));
    }
  }


  /**
   * Resume the instance that have the specified Id on the orchestra server.
   *
   * @author Mathieu Crassin
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  public final void resumeInstance(final String instanceId) throws OrchestraManagementException {
    // check if the API fields are null or not
    if (this.instanceManagementAPI == null) {
      this.getAPIs();
    }
    try {
      final ProcessInstanceUUID inst = new ProcessInstanceUUID(instanceId);
      this.instanceManagementAPI.resume(inst);
    } catch (final InstanceNotFoundException e) {
      Misc.log(Level.SEVERE,"OrchestraManagement - resumeInstance :"
          + "Error, instance " + instanceId + " not found !\n" + Misc.getStackTraceFrom(e));
      throw new OrchestraManagementException("Error while resuming the instance with uuid " + instanceId + " !",
          "OrchestraManagement - resumeInstance : Error, instance not found !\n" + Misc.getStackTraceFrom(e));
    } catch (final Exception ex) {
      Misc.log(Level.SEVERE,"OrchestraManagement - resumeInstance :"
          + "Error while resuming instance : " + instanceId + " !\n" + Misc.getStackTraceFrom(ex));
      throw new OrchestraManagementException("Error while resuming the instance with uuid " + instanceId + " !",
          "OrchestraManagement - resumeInstance : \n" + Misc.getStackTraceFrom(ex));
    }
  }


  /**
   * Exit the instance that have the specified Id on the orchestra server.
   *
   * @author Mathieu Crassin
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  public final void exitInstance(final String instanceId) throws OrchestraManagementException {
    // check if the API fields are null or not
    if (this.instanceManagementAPI == null) {
      this.getAPIs();
    }
    try {
      this.instanceManagementAPI.exit(new ProcessInstanceUUID(instanceId));
    } catch (final InstanceNotFoundException e) {
      Misc.log(Level.SEVERE,"OrchestraManagement - exitInstance :"
          + "Error, instance " + instanceId + " not found !\n" + Misc.getStackTraceFrom(e));
      throw new OrchestraManagementException("Error while exiting the instance with uuid " + instanceId + " !",
          "OrchestraManagement - exitInstance : Error, instance not found !\n" + Misc.getStackTraceFrom(e));
    } catch (final Exception ex) {
      Misc.log(Level.SEVERE,"OrchestraManagement - exitInstance :"
          + "Error while exiting instance : " + instanceId + " !\n" + Misc.getStackTraceFrom(ex));
      throw new OrchestraManagementException("Error while exiting the instance with uuid " + instanceId + " !",
          "OrchestraManagement - exitInstance : \n" + Misc.getStackTraceFrom(ex));
    }
  }


  /**
   * Delete the instance that have the specified Id on the orchestra server.
   *
   * @author Mathieu Crassin
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  public void removeInstance(final String instanceId) throws OrchestraManagementException {
    // check if the API fields are null or not
    if (this.instanceManagementAPI == null) {
      this.getAPIs();
    }

    // call the function
    boolean res = false;
    try {
      res = this.instanceManagementAPI.remove(new ProcessInstanceUUID(instanceId));
    } catch (final Exception ex) {
      Misc.log(Level.SEVERE,"OrchestraManagement - removeInstance : "
          + "Error while removing the instance with uuid " + instanceId + " !" + Misc.getStackTraceFrom(ex));
      throw new OrchestraManagementException("Error while removing the instance with id " + instanceId + " !",
          "OrchestraManagement - removeInstance : \n" + Misc.getStackTraceFrom(ex));
    }

    // check the result
    if (!res) {
      Misc.log(Level.SEVERE,"OrchestraManagement - removeInstance :"
          + "Fail to remove instance with id = " + instanceId + ", return result was wrong !");
      throw new OrchestraManagementException("Fail to remove instance with id = " + instanceId,
          "OrchestraManagement - removeInstance : Fail to remove instance with id = "
          + instanceId + ", return result was wrong !");
    }
  }


  /**
   * Remove definitively a process from the orchestra server.
   *
   * @author Mathieu Crassin
   * @param processId : Id of the process to remove.
   */
  public void removeProcess(final String processId) throws OrchestraManagementException {
    // check if the API fields are null or not
    if (this.managementAPI == null) {
      this.getAPIs();
    }

    // then call the remove function
    boolean res = false;
    try {
      res = this.managementAPI.remove(new ProcessDefinitionUUID(processId));
    } catch (final Exception ex) {
      Misc.log(Level.SEVERE,"OrchestraManagement - removeProcess : "
          + "Error while removing the process with uuid " + processId + " !" + Misc.getStackTraceFrom(ex));
      throw new OrchestraManagementException("Error while removing the process with uuid " + processId + " !",
          "OrchestraManagement - removeProcess : \n" + Misc.getStackTraceFrom(ex));
    }

    // check the result
    if (!res) {
      Misc.log(Level.SEVERE,"OrchestraManagement - removeProcess :"
          + "Fail to remove process with id = " + processId + ", return result was wrong !");
      throw new OrchestraManagementException("Fail to remove process with id = " + processId,
          "OrchestraManagement - removeProcess : Fail to remove process with id = " + processId + ", return result was wrong !");
    }
  }


  /**
   * Get the activities of a given process
   *
   * @author Jean-Yves Goubet
   * @param processId : it's the UUID of the process
   * @return List<MProcessActivity> : The list filled with process' activities
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  public final List<MProcessActivity> getProcessActivities(final String processId) throws OrchestraManagementException {
    final List<MProcessActivity> mProcessActivityList = new ArrayList<MProcessActivity>();
    Set<ActivityDefinition> processActivitySet = null;

    // check if the API fields are null or not
    if (this.querierDefinitionAPI == null) {
      this.getAPIs();
    }

    // get the instance activities set from orchestra
    try {
      processActivitySet = this.querierDefinitionAPI.findActivityDefinitions(new ProcessDefinitionUUID(processId));
    } catch (final Exception ex) {
      Misc.log(Level.SEVERE, "Error while sending request to orchestra server !");
      throw new OrchestraManagementException("Error while sending request to orchestra server !",
          "OrchestraManagement - getInstanceActivities : \n" + Misc.getStackTraceFrom(ex));
    }

    // fill in the list with appropriate values
    if (processActivitySet != null) {
      for (final ActivityDefinition a : processActivitySet) {
        mProcessActivityList.add(new MProcessActivity(a.getName(), a.getType().toString()));
      }
    } else {
      Misc.log(Level.SEVERE,"OrchestraManagement - getProcessActivities :"
          + "Unable to get process activities on orchestra server !\nThe return result of orchestra is null.");
      throw new OrchestraManagementException("Unable to get process activities list on orchestra server !",
          "OrchestraManagement - getProcessActivities : The return result of orchestra is null.");
    }

    return mProcessActivityList;
  }


  /**
   * Get the activities of a given instance
   *
   * @author Jean-Yves Goubet
   * @param instanceId : it's the UUID of the instance
   * @return List<MInstanceActivity> : The list filled with instance' activities
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  public final List<MInstanceActivity> getInstanceActivities(final String instanceId) throws OrchestraManagementException {
    final List<MInstanceActivity> mInstanceActivityList = new ArrayList<MInstanceActivity>();
    Set<ActivityInstance> instanceActivitySet = null;

    if (instanceId == null || instanceId.equals("")) {
      return mInstanceActivityList;
    }

    // check if the API fields are null or not
    if (this.querierRuntimeAPI == null) {
      this.getAPIs();
    }

    // get the activity instance set from orchestra
    try {
      instanceActivitySet = this.querierRuntimeAPI.findActivityInstances(new ProcessInstanceUUID(instanceId));
    } catch (final Exception ex) {
      Misc.log(Level.SEVERE, "Error while sending request to orchestra server !");
      throw new OrchestraManagementException("Error while sending request to orchestra server !",
          "OrchestraManagement - getInstanceActivities : \n" + Misc.getStackTraceFrom(ex));
    }

    // fill in the list with appropriate values
    if (instanceActivitySet != null) {
      String activityName = null;
      for (final ActivityInstance a : instanceActivitySet) {
        try {
          activityName = this.querierDefinitionAPI.getActivityDefinition(a.getDefinitionUUID()).getName();
        } catch (final Exception ex) {
          Misc.log(Level.SEVERE, "Error while sending request to orchestra server !");
          throw new OrchestraManagementException("Error while sending request to orchestra server !",
              "OrchestraManagement - getInstanceActivities (getActivityName()): \n" + Misc.getStackTraceFrom(ex));
        }
        mInstanceActivityList.add(new MInstanceActivity(a.getUUID().toString(), activityName, a.getType().toString(),
            a.getState().toString(), a.getStartedDate(), a.getEndedDate()));
      }
    } else {
      Misc.log(Level.SEVERE,"OrchestraManagement - getInstanceActivities :"
          + "Unable to get instance activities on orchestra server !\nThe return result of orchestra is null.");
      throw new OrchestraManagementException("Unable to get instance activities list on orchestra server !",
          "OrchestraManagement - getInstanceActivities : The return result of orchestra is null.");
    }

    return mInstanceActivityList;
  }


  /**
   * Get the entire content of a file.
   *
   * @author Mathieu Crassin
   * @param filename
   * @return
   * @throws OrchestraManagementException
   */
  protected byte[] getContentFile(final String filename) throws OrchestraManagementException {
    byte []buffer = null;
    try {
      // create file object
      final File file = new File(filename);
      //create FileInputStream object
      final FileInputStream fis = new FileInputStream(file);
      buffer = new byte[(int) file.length()];
      fis.read(buffer);
      fis.close();
      // display
      Misc.log(Level.INFO,"OrchestraManagement - getContentFile : File " + filename + " found !");
      // and try to delete it
      if (file.delete()) {
        Misc.log(Level.INFO,"OrchestraManagement - getContentFile : Temporary file " + filename + " deleted");
      } else {
        Misc.log(Level.SEVERE,"OrchestraManagement - getContentFile : Error while deleting file " + filename);
        throw new OrchestraManagementException("Internal error during connexion with Orchestra  !",
            "OrchestraManagement - getContentFile : Error while deleting file " + filename);
      }
    } catch (final NullPointerException e) {
      Misc.log(Level.SEVERE,"OrchestraManagement - getContentFile :"
          + "The file name is a null pointer !\n" + Misc.getStackTraceFrom(e));
      throw new OrchestraManagementException("Internal server error while accessing file !",
          "OrchestraManagement - getContentFile : The file name is a null pointer !\n" + Misc.getStackTraceFrom(e));
    } catch (final FileNotFoundException e) {
      Misc.log(Level.SEVERE,"OrchestraManagement - getContentFile : File not found !\n" + Misc.getStackTraceFrom(e));
      throw new OrchestraManagementException("Internal server error while accessing file !",
          "OrchestraManagement - getContentFile : File not found !\n" + Misc.getStackTraceFrom(e));
    } catch (final IOException ioe) {
      Misc.log(Level.SEVERE,"OrchestraManagement - getContentFile :"
          + "Exception while reading the file !\n" + Misc.getStackTraceFrom(ioe));
      throw new OrchestraManagementException("Internal server error while accessing file !",
          "OrchestraManagement - getContentFile : Exception while reading the file !\n" + Misc.getStackTraceFrom(ioe));
    } catch (final SecurityException se) {
      Misc.log(Level.SEVERE,"OrchestraManagement - getContentFile :"
          + "Exception while deleting file !\n" + Misc.getStackTraceFrom(se));
      throw new OrchestraManagementException("Internal server error while accessing file !",
          "OrchestraManagement - getContentFile : Exception while deleting file !\n" + Misc.getStackTraceFrom(se));
    }

    return buffer;
  }


  /**
   * This method is used to fill in the APIs field of this class
   *
   * @author Jean-Yves Goubet
   * @throws OrchestraManagementException
   */
  protected void getAPIs() throws OrchestraManagementException {
    try {
      this.querierRuntimeAPI = AccessorUtil.getQueryRuntimeAPI(this.serviceUrl, this.objectName);
      this.querierDefinitionAPI = AccessorUtil.getQueryDefinitionAPI(this.serviceUrl, this.objectName);
      this.instanceManagementAPI = AccessorUtil.getInstanceManagementAPI(this.serviceUrl, this.objectName);
      this.managementAPI = AccessorUtil.getManagementAPI(this.serviceUrl, this.objectName);
    } catch (final Exception e) {
      Misc.log(Level.SEVERE, "Error while accessing to orchestra server !\n" + Misc.getStackTraceFrom(e));
      throw new OrchestraManagementException("Error while accessing to orchestra server !",
                                             "OrchestraManagement - getAPIs :\n" + Misc.getStackTraceFrom(e));
    }
    if (this.querierRuntimeAPI == null || this.querierDefinitionAPI == null || this.instanceManagementAPI == null || this.managementAPI == null) {
      Misc.log(Level.SEVERE, "Error while accessing to orchestra server !\nThe return result of orchestra is null.");
      throw new OrchestraManagementException("Error while accessing to orchestra server !",
          "OrchestraManagement - getAPIs : The return result of orchestra is null.");
    }
  }


  /**
   * This method is used to get the service url and object name properties from conf file orchestra.properties
   * It first checks if the conf/orchestra.properties exists to use it, otherwise it uses the file included in the war
   * to fill the fields service url and object name of this class
   *
   * @author Jean-Yves Goubet
   * @throws OrchestraManagementException
   */
  public void getOrchestraProperties() throws OrchestraManagementException {
    InputStream inputStream = null;
    try {
      final File file = new File("conf/orchestra.properties");
      final Properties properties = new Properties();
      if (file.exists()) {
        inputStream = new FileInputStream(file);
      } else {
        inputStream = Thread.currentThread().getContextClassLoader().getResourceAsStream("orchestra.properties");
      }
      properties.load(inputStream);
      this.serviceUrl = properties.getProperty("orchestra.jmx.serviceUrl");
      this.objectName = properties.getProperty("orchestra.jmx.objectName");

    } catch (final IOException e) {
      Misc.log(Level.SEVERE,"OrchestraManagement - getOrchestraProperties :"
          + "Unable to load property file !\n" + Misc.getStackTraceFrom(e));
      throw new OrchestraManagementException("Unable to load property file !",
          "OrchestraManagement - getOrchestraProperties : Unable to load property file !\n" + Misc.getStackTraceFrom(e));
    } finally {
      Misc.close(inputStream);
    }
  }


  /**
   * This function return the process name corresponding to an uuid.
   * First the function check if a correspondence is registered in the list, in this case,
   * the response can be directly return
   * in the other case, the response is ask to orchestra server.
   *
   * @param correspondenceList
   * @param uuid
   * @return
   * @throws OrchestraManagementException
   */
  protected String searchProcessName(final List<ProcessUuidNName> correspondenceList,
      final ProcessDefinitionUUID uuid) throws OrchestraManagementException {

    if (uuid == null) {
      Misc.log(Level.SEVERE, "Error while accessing process informations");
      throw new OrchestraManagementException("Error while accessing process informations",
          "OrchestraManagement - searchProcessName : Parameter uuid is null");
    }

    String processName = "error";
    int res = -1;
    if (correspondenceList != null) {
      res = OrchestraManagementServlet.correspondenceExists(correspondenceList, uuid.toString());
    }

    // if we didn't find a correspondence in the list
    if (res == -1) {
//System.out.println("ask orchestra for uuid : " + uuid.toString());

      // check if the API fields are null or not
      if (this.querierDefinitionAPI == null) {
        this.getAPIs();
      }

      ProcessDefinition processDef;
      try {
        processDef = this.querierDefinitionAPI.getProcessDefinition(uuid);
      } catch (final ProcessNotFoundException e) {
        Misc.log(Level.SEVERE, "Error while accessing process (id = " + uuid.toString() + " ) "
            + "The process was not found on orchestra !");
        throw new OrchestraManagementException("Error while accessing process (id = " + uuid.toString() + " )"
            + "The process was not found on orchestra !",
            "OrchestraManagement - searchProcessName : " + Misc.getStackTraceFrom(e));
      } catch (final Exception e) {
        Misc.log(Level.SEVERE, "Error while accessing process (id = " + uuid.toString() + " )");
        throw new OrchestraManagementException("Error while accessing process (id = " + uuid.toString() + " )",
            "OrchestraManagement - searchProcessName : " + Misc.getStackTraceFrom(e));
      }

      if (processDef != null) {
        processName = processDef.getName();
      } else {
        Misc.log(Level.SEVERE, "Error while accessing process (id = " + uuid.toString() + " )");
        throw new OrchestraManagementException("Error while accessing process (id = " + uuid.toString() + " )",
            "OrchestraManagement - searchProcessName : The orchestra returned object is null");
      }

      // add the correspondence in the grid
      if (correspondenceList != null) {
        correspondenceList.add(new ProcessUuidNName(uuid.toString(), processName));
      }
    }
    // if we found a correspondence in the list
    else {
//System.out.println("found uuid : " + uuid.toString() + " in our table");

      final ProcessUuidNName tmp = correspondenceList.get(res);
      processName = tmp.getProcessName();

    }

    return processName;
  }


  /**
   * The class use to construct the correspondence list between the process uuid
   * and the process name.
   *
   * @author Mathieu Crassin
   */
  protected class ProcessUuidNName {
    private String processUUID = null;
    private String processName = null;

    public ProcessUuidNName(final String processUUID, final String processName) {
      this.processUUID = processUUID;
      this.processName = processName;
    }

    public String getProcessUUID() {
      return this.processUUID;
    }
    public String getProcessName() {
      return this.processName;
    }

  }


  /**
   * This method scan a list to check if a correspondence with a given uuid is here
   *
   * @author Mathieu Crassin
   * @param correspondenceList
   * @param processUUID
   * @return the index of the uuid, if exists, or -1
   */
  protected static int correspondenceExists(final List<ProcessUuidNName> correspondenceList, final String processUUID) {
    int i = 0;
    for (final ProcessUuidNName processUuidNName : correspondenceList) {
      if (processUuidNName.getProcessUUID().equals(processUUID)) {
        return i;
      }
      i++;
    }

    return -1;
  }


  public void sessionLogout() throws OrchestraManagementException {
    this.getThreadLocalRequest().getSession().invalidate();
  }

  public String getSessionUserName() throws OrchestraManagementException {
    return this.getThreadLocalRequest().getRemoteUser();
  }

  public long getSessionCreationDate() throws OrchestraManagementException {
    return this.getThreadLocalRequest().getSession().getCreationTime();
  }

}
